@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+from coredis._utils import b
 from coredis.typing import Callable, ClassVar, Dict, Tuple, ValueT
 
 
@@ -74,6 +75,7 @@ class KeySpec:
         b"TOUCH": lambda args: (args[1 : (len(args))]),
         b"TTL": lambda args: ((args[1],)),
         b"TYPE": lambda args: ((args[1],)),
+        b"WATCH": lambda args: (args[1 : (len(args))]),
         b"XINFO CONSUMERS": lambda args: ((args[1],)),
         b"XINFO GROUPS": lambda args: ((args[1],)),
         b"XINFO STREAM": lambda args: ((args[1],)),
@@ -129,6 +131,7 @@ class KeySpec:
     }
     ALL: ClassVar[Dict[bytes, Callable[[Tuple[ValueT, ...]], Tuple[ValueT, ...]]]] = {
         b"OBJECT": lambda args: ((args[2],)),
+        b"DEBUG OBJECT": lambda args: ((args[1],)),
         b"APPEND": lambda args: ((args[1],)),
         b"BITFIELD": lambda args: ((args[1],)),
         b"BLMOVE": lambda args: ((args[1],) + (args[2],)),
@@ -310,6 +313,7 @@ class KeySpec:
         b"TOUCH": lambda args: (args[1 : (len(args))]),
         b"TTL": lambda args: ((args[1],)),
         b"TYPE": lambda args: ((args[1],)),
+        b"WATCH": lambda args: (args[1 : (len(args))]),
         b"XINFO CONSUMERS": lambda args: ((args[1],)),
         b"XINFO GROUPS": lambda args: ((args[1],)),
         b"XINFO STREAM": lambda args: ((args[1],)),
@@ -374,7 +378,6 @@ class KeySpec:
         b"SSUBSCRIBE": lambda args: (args[1 : (len(args))]),
         b"SUNSUBSCRIBE": lambda args: (args[1 : (len(args))]),
         b"UNLINK": lambda args: (args[1 : (len(args))]),
-        b"WATCH": lambda args: (args[1 : (len(args))]),
         b"PUBLISH": lambda args: ((args[1],)),
         b"EXPIREMEMBER": lambda args: ((args[1],)),
         b"EXPIREMEMBERAT": lambda args: ((args[1],)),
@@ -391,9 +394,7 @@ class KeySpec:
         if len(arguments) <= 1:
             return ()
 
-        command = arguments[0]
-        if not isinstance(command, bytes):
-            command = str(command).encode("latin-1")
+        command = b(arguments[0])
 
         try:
             if readonly_command and command in cls.READONLY:
