@@ -5,14 +5,15 @@ import dataclasses
 import functools
 import random
 import textwrap
+import warnings
 from typing import TYPE_CHECKING, Any, cast
 
 from packaging import version
 
-from coredis.cache import AbstractCache
+from coredis.cache import AbstractCache, SupportsSampling
 from coredis.commands._utils import check_version, redis_command_link
-from coredis.commands.constants import CommandGroup, CommandName
-from coredis.nodemanager import NodeFlag
+from coredis.commands.constants import CommandFlag, CommandGroup, CommandName, NodeFlag
+from coredis.globals import COMMAND_FLAGS, READONLY_COMMANDS
 from coredis.response._callbacks import ClusterMultiNodeCallback
 from coredis.typing import (
     AsyncIterator,
@@ -24,6 +25,8 @@ from coredis.typing import (
     P,
     R,
     ResponseType,
+    Set,
+    add_runtime_checks,
 )
 
 if TYPE_CHECKING:
@@ -35,15 +38,20 @@ class CacheConfig:
     key_func: Callable[..., bytes]
 
 
+class RedirectUsage(NamedTuple):
+    reason: str
+    warn: bool
+
+
 class CommandDetails(NamedTuple):
     command: bytes
     group: Optional[CommandGroup]
-    readonly: bool
     version_introduced: Optional[version.Version]
     version_deprecated: Optional[version.Version]
     arguments: Dict[str, Dict[str, str]]
     cluster: ClusterCommandConfig
     cache_config: Optional[CacheConfig]
+    flags: Set[CommandFlag]
 
 
 @dataclasses.dataclass
@@ -76,7 +84,9 @@ class CommandCache:
     ) -> AsyncIterator[R]:
         client = args[0]
         cache = getattr(client, "cache")
-        if not (self.cache_config and cache):
+        noreply = getattr(client, "noreply")
+
+        if not (self.cache_config and cache) or noreply:
             yield await func(*args, **kwargs)
         else:
             assert isinstance(cache, AbstractCache)
@@ -91,7 +101,9 @@ class CommandCache:
                             self.command, key, *args[1:], *kwargs.items()  # type: ignore
                         ),
                     )
-                    if not random.random() * 100.0 < min(100.0, cache.confidence):
+                    if isinstance(
+                        cache, SupportsSampling
+                    ) and not random.random() * 100.0 < min(100.0, cache.confidence):
                         actual = await func(*args, **kwargs)
                         cache.feedback(
                             self.command,
@@ -121,43 +133,67 @@ def redis_command(
     version_introduced: Optional[str] = None,
     version_deprecated: Optional[str] = None,
     deprecation_reason: Optional[str] = None,
+    redirect_usage: Optional[RedirectUsage] = None,
     arguments: Optional[Dict[str, Dict[str, str]]] = None,
-    readonly: bool = False,
+    flags: Optional[Set[CommandFlag]] = None,
     cluster: ClusterCommandConfig = ClusterCommandConfig(),
     cache_config: Optional[CacheConfig] = None,
 ) -> Callable[
     [Callable[P, Coroutine[Any, Any, R]]], Callable[P, Coroutine[Any, Any, R]]
 ]:
-    if not readonly and cache_config:
-        raise RuntimeError()
+    readonly = False
+    if flags and CommandFlag.READONLY in flags:
+        READONLY_COMMANDS.add(command_name)
+        readonly = True
+
+    if not readonly and cache_config:  # noqa
+        raise RuntimeError(
+            f"Can't decorate non readonly command {command_name} with cache config"
+        )
+
+    COMMAND_FLAGS[command_name] = flags or set()
 
     command_details = CommandDetails(
         command_name,
         group,
-        readonly,
         version.Version(version_introduced) if version_introduced else None,
         version.Version(version_deprecated) if version_deprecated else None,
         arguments or {},
         cluster or ClusterCommandConfig(),
         cache_config,
+        flags or set(),
     )
 
     def wrapper(
         func: Callable[P, Coroutine[Any, Any, R]]
     ) -> Callable[P, Coroutine[Any, Any, R]]:
         command_cache = CommandCache(command_name, cache_config)
+        runtime_checkable = add_runtime_checks(func)
 
         @functools.wraps(func)
         async def wrapped(*args: P.args, **kwargs: P.kwargs) -> R:
+            if redirect_usage:
+                if redirect_usage.warn:
+                    warnings.warn(redirect_usage.reason, UserWarning, stacklevel=2)
+                else:
+                    raise NotImplementedError(redirect_usage.reason)
+
+            from coredis.client import Redis, RedisCluster
+
+            client = args[0]
+            runtime_checking = not getattr(client, "noreply", None) and isinstance(
+                client, (Redis, RedisCluster)
+            )
+            callable = runtime_checkable if runtime_checking else func
             check_version(
-                args[0],  # type: ignore
+                client,  # type: ignore
                 command_name,
                 func.__name__,
                 command_details.version_introduced,
                 command_details.version_deprecated,
                 deprecation_reason,
             )
-            async with command_cache(func, *args, **kwargs) as response:
+            async with command_cache(callable, *args, **kwargs) as response:
                 return response
 
         wrapped.__doc__ = textwrap.dedent(wrapped.__doc__ or "")
@@ -184,6 +220,17 @@ Deprecated in :redis-version:`{version_deprecated}`
             wrapped.__doc__ += """
 Supports client side caching
             """
+        if redirect_usage:
+            if redirect_usage.warn:
+                preamble = f".. warning:: Using ``{func.__name__}`` directly is not recommended."
+            else:
+                preamble = (
+                    f".. danger:: Using ``{func.__name__}`` directly is not supported."
+                )
+            wrapped.__doc__ += f"""
+{preamble} {redirect_usage.reason}
+"""
+
         setattr(wrapped, "__coredis_command", command_details)
         return wrapped
 
