@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import asyncio
 import functools
 import inspect
 import sys
@@ -13,10 +14,10 @@ from typing import Any, cast
 
 from wrapt import ObjectProxy  # type: ignore
 
-from coredis._utils import clusterdown_wrapper
-from coredis.client import AbstractRedis
+from coredis._utils import b, clusterdown_wrapper, hash_slot
+from coredis.client import Client
 from coredis.commands._key_spec import KeySpec
-from coredis.commands.constants import CommandName
+from coredis.commands.constants import CommandName, NodeFlag
 from coredis.commands.script import Script
 from coredis.connection import BaseConnection, ClusterConnection
 from coredis.exceptions import (
@@ -33,8 +34,8 @@ from coredis.exceptions import (
     TryAgainError,
     WatchError,
 )
-from coredis.nodemanager import Node, NodeFlag
 from coredis.pool import ClusterConnectionPool, ConnectionPool
+from coredis.pool.nodemanager import ManagedNode
 from coredis.response._callbacks import (
     AnyStrCallback,
     BoolCallback,
@@ -105,6 +106,7 @@ class PipelineCommand:
     args: Tuple[ValueT, ...]
     callback: Callable[..., Any] = NoopCallback()  # type: ignore
     options: Dict[str, Optional[ValueT]] = field(default_factory=dict)
+    request: Optional[asyncio.Future[ResponseType]] = None
 
 
 @dataclass
@@ -117,16 +119,12 @@ class ClusterPipelineCommand(PipelineCommand):
 class NodeCommands:
     def __init__(
         self,
-        parse_response: Callable[..., Coroutine[Any, Any, ResponseType]],
-        response_callbacks: Dict[bytes, Callable[..., Any]],
         connection: ClusterConnection,
         in_transaction: bool = False,
     ):
-        self.parse_response = parse_response
         self.connection = connection
         self.commands: List[ClusterPipelineCommand] = []
         self.in_transaction = in_transaction
-        self.response_callbacks = response_callbacks
 
     def extend(self, c: List[ClusterPipelineCommand]) -> None:
         self.commands.extend(c)
@@ -135,7 +133,6 @@ class NodeCommands:
         self.commands.append(c)
 
     async def write(self) -> None:
-        """NOTE: Code was borrowed from Redis so it can be fixed"""
         connection = self.connection
         commands = self.commands
 
@@ -148,9 +145,10 @@ class NodeCommands:
         # build up all commands into a single request to increase network perf
         # send all the commands and catch connection and timeout errors.
         try:
-            await connection.send_packed_command(
-                connection.pack_commands([(c.command,) + c.args for c in commands])
-            )
+            for cmd in commands:
+                cmd.request = await connection.create_request(
+                    cmd.command, *cmd.args, decode=cmd.options.get("decode")
+                )
         except (ConnectionError, TimeoutError) as e:
             for c in commands:
                 c.result = e
@@ -160,54 +158,44 @@ class NodeCommands:
         success = True
 
         for c in self.commands:
-
-            # if there is a result on this command, it means we ran into an exception
-            # like a connection error. Trying to parse a response on a connection that
-            # is no longer open will result in a connection error raised by redis-py.
-            # but redis-py doesn't check in parse_response that the sock object is
-            # still set and if you try to read from a closed connection, it will
-            # result in an AttributeError because it will do a readline() call on None.
-            # This can have all kinds of nasty side-effects.
-            # Treating this case as a connection error is fine because it will dump
-            # the connection object back into the pool and on the next write, it will
-            # explicitly open the connection and all will be well.
-
             if c.result is None:
                 try:
-                    if self.in_transaction:
-                        cmd = "_"
-                    else:
-                        cmd = c.command
-                    c.result = await self.parse_response(connection, cmd, **c.options)
+                    c.result = await c.request if c.request else None
                 except ExecAbortError:
                     raise
                 except (ConnectionError, TimeoutError) as e:
                     success = False
                     for c in self.commands:
                         c.result = e
+                    break
                 except RedisError:
                     success = False
                     c.result = sys.exc_info()[1]
+                    break
 
         if self.in_transaction:
             transaction_result = []
             if success:
                 for c in self.commands:
                     if c.command == CommandName.EXEC:
-                        transaction_result = cast(List[ResponseType], c.result)
-            for idx, c in enumerate(
-                [
-                    _c
-                    for _c in sorted(self.commands, key=lambda x: x.position)
-                    if _c.command not in {CommandName.MULTI, CommandName.EXEC}
-                ]
-            ):
-                callback = self.response_callbacks.get(c.command, c.callback)
-                c.result = callback(
-                    transaction_result[idx],
-                    version=connection.protocol_version,
-                    **c.options,
-                )
+                        if c.result:
+                            transaction_result = cast(List[ResponseType], c.result)
+                        else:
+                            raise WatchError()
+                for idx, c in enumerate(
+                    [
+                        _c
+                        for _c in sorted(self.commands, key=lambda x: x.position)
+                        if _c.command not in {CommandName.MULTI, CommandName.EXEC}
+                    ]
+                ):
+                    c.result = c.callback(
+                        transaction_result[idx],
+                        version=connection.protocol_version,
+                        **c.options,
+                    )
+            elif isinstance(self.commands[0].result, BaseException):
+                raise self.commands[0].result
 
 
 class PipelineMeta(ABCMeta):
@@ -232,7 +220,8 @@ class ClusterPipelineMeta(PipelineMeta):
     def __new__(cls, name: str, bases: Tuple[type, ...], namespace: Dict[str, object]):
         kls = super().__new__(cls, name, bases, namespace)
         for name, method in ClusterPipelineMeta.get_methods(kls).items():
-            if cmd := getattr(method, "__coredis_command", None):
+            cmd = getattr(method, "__coredis_command", None)
+            if cmd:
                 if cmd.cluster.route:
                     kls.NODES_FLAGS[cmd.command] = cmd.cluster.route
                 if cmd.cluster.multi_node:
@@ -246,7 +235,7 @@ class ClusterPipelineMeta(PipelineMeta):
         return kls
 
 
-class PipelineImpl(AbstractRedis[AnyStr], metaclass=PipelineMeta):
+class PipelineImpl(Client[AnyStr], metaclass=PipelineMeta):
     """Pipeline for the Redis class"""
 
     """
@@ -273,15 +262,18 @@ class PipelineImpl(AbstractRedis[AnyStr], metaclass=PipelineMeta):
     def __init__(
         self,
         connection_pool: ConnectionPool,
-        response_callbacks: Dict[bytes, Callable[..., Any]],
         transaction: Optional[bool],
+        watches: Optional[Parameters[KeyT]] = None,
     ) -> None:
         self.connection_pool = connection_pool
         self.connection = None
-        self.response_callbacks = response_callbacks
         self._transaction = transaction
         self.watching = False
+        self.watches: Optional[Parameters[KeyT]] = watches or None
         self.command_stack = []
+        self.cache = None  # not implemented.
+        self.explicit_transaction = False
+        self.scripts: Set[Script[AnyStr]] = set()
 
     async def __aenter__(self) -> "PipelineImpl[AnyStr]":
         return self
@@ -310,13 +302,16 @@ class PipelineImpl(AbstractRedis[AnyStr], metaclass=PipelineMeta):
             try:
                 # call this manually since our unwatch or
                 # immediate_execute_command methods can call reset_pipeline()
-                await self.connection.send_command(CommandName.UNWATCH)
-                await self.connection.read_response()
+                request = await self.connection.create_request(
+                    CommandName.UNWATCH, decode=False
+                )
+                await request
             except ConnectionError:
                 # disconnect will also remove any previous WATCHes
                 self.connection.disconnect()
         # clean up the other instance attributes
         self.watching = False
+        self.watches = []
         self.explicit_transaction = False
         # we can safely return the connection to the pool here since we're
         # sure we're no longer WATCHing anything
@@ -380,11 +375,14 @@ class PipelineImpl(AbstractRedis[AnyStr], metaclass=PipelineMeta):
             conn = await self.connection_pool.get_connection()
             self.connection = conn
         try:
-            await conn.send_command(command, *args)
+            request = await conn.create_request(
+                command, *args, decode=kwargs.get("decode")
+            )
 
             return callback(
-                await self.parse_response(conn, command, **kwargs),
+                await request,
                 version=conn.protocol_version,
+                **kwargs,
             )
         except (ConnectionError, TimeoutError) as e:
             conn.disconnect()
@@ -394,17 +392,22 @@ class PipelineImpl(AbstractRedis[AnyStr], metaclass=PipelineMeta):
             # if we're not already watching, we can safely retry the command
             try:
                 if not self.watching:
-                    await conn.send_command(command, *args)
-
+                    request = await conn.create_request(
+                        command, *args, decode=kwargs.get("decode")
+                    )
                     return callback(
-                        await self.parse_response(conn, command, **kwargs),
-                        version=conn.protocol_version,
+                        await request, version=conn.protocol_version, **kwargs
                     )
             except ConnectionError:
                 # the retry failed so cleanup.
                 conn.disconnect()
                 await self.reset_pipeline()
                 raise
+        finally:
+            if command in self.UNWATCH_COMMANDS:
+                self.watching = False
+            elif command == CommandName.WATCH:
+                self.watching = True
 
     def pipeline_execute_command(
         self,
@@ -441,43 +444,62 @@ class PipelineImpl(AbstractRedis[AnyStr], metaclass=PipelineMeta):
         raise_on_error: bool,
     ) -> Tuple[Any, ...]:
 
-        cmds = chain(
-            [PipelineCommand(command=CommandName.MULTI, args=())],
-            commands,
-            [PipelineCommand(command=CommandName.EXEC, args=())],
+        cmds = list(
+            chain(
+                [
+                    PipelineCommand(
+                        command=CommandName.MULTI,
+                        args=(),
+                    )
+                ],
+                commands,
+                [
+                    PipelineCommand(
+                        command=CommandName.EXEC,
+                        args=(),
+                    )
+                ],
+            )
         )
-        all_cmds = connection.pack_commands([(cmd.command,) + cmd.args for cmd in cmds])
-        await connection.send_packed_command(all_cmds)
+        if self.watches:
+            await self.watch(*self.watches)
+
+        for cmd in cmds:
+            cmd.request = await connection.create_request(
+                cmd.command, *cmd.args, decode=cmd.options.get("decode")
+            )
+
         errors: List[Tuple[int, Optional[RedisError]]] = []
+        multi_failed = False
 
         # parse off the response for MULTI
         # NOTE: we need to handle ResponseErrors here and continue
         # so that we read all the additional command messages from
         # the socket
         try:
-            await self.parse_response(connection, b"_")
+            await cmds[0].request if cmds[0].request else None
         except RedisError:
+            multi_failed = True
             errors.append((0, cast(RedisError, sys.exc_info()[1])))
 
         # and all the other commands
-
-        for i, cmd in enumerate(commands):
+        for i, cmd in enumerate(cmds[1:-1]):
             try:
-                await self.parse_response(connection, b"_")
+                if cmd.request:
+                    assert (await cmd.request) in {b"QUEUED", "QUEUED"}
             except RedisError:
                 ex = cast(RedisError, sys.exc_info()[1])
                 self.annotate_exception(ex, i + 1, cmd.command, cmd.args)
                 errors.append((i, ex))
 
-        # parse the EXEC.
         response: List[ResponseType]
-
         try:
             response = cast(
-                List[ResponseType], await self.parse_response(connection, b"_")
+                List[ResponseType],
+                await cmds[-1].request if cmds[-1].request else None,
             )
-        except ExecAbortError:
-            if self.explicit_transaction:
+        except (ExecAbortError, ResponseError):
+            if self.explicit_transaction and not multi_failed:
                 await self.immediate_execute_command(
                     CommandName.DISCARD, callback=BoolCallback()
                 )
@@ -492,7 +514,7 @@ class PipelineImpl(AbstractRedis[AnyStr], metaclass=PipelineMeta):
         # put any parse errors into the response
 
         for i, e in errors:
-            response.insert(i, e)
+            response.insert(i, cast(ResponseType, e))
 
         if len(response) != len(commands):
             if self.connection:
@@ -502,23 +524,15 @@ class PipelineImpl(AbstractRedis[AnyStr], metaclass=PipelineMeta):
             )
 
         # find any errors in the response and raise if necessary
-
         if raise_on_error:
             self.raise_first_error(commands, response)
 
         # We have to run response callbacks manually
         data: List[Any] = []
-
         for r, cmd in zip(response, commands):
             if not isinstance(r, Exception):
-
-                callback = self.response_callbacks.get(cmd.command, cmd.callback)
-                r = callback(r, version=connection.protocol_version, **cmd.options)
-                if inspect.isawaitable(response):
-                    r = await r
-
+                r = cmd.callback(r, version=connection.protocol_version, **cmd.options)
             data.append(r)
-
         return tuple(data)
 
     async def _execute_pipeline(
@@ -528,10 +542,10 @@ class PipelineImpl(AbstractRedis[AnyStr], metaclass=PipelineMeta):
         raise_on_error: bool,
     ) -> Tuple[Any, ...]:
         # build up all commands into a single request to increase network perf
-        all_cmds = connection.pack_commands(
-            [(cmd.command,) + cmd.args for cmd in commands]
-        )
-        await connection.send_packed_command(all_cmds)
+        for cmd in commands:
+            cmd.request = await connection.create_request(
+                cmd.command, *cmd.args, decode=cmd.options.get("decode")
+            )
 
         response: List[Any] = []
 
@@ -539,9 +553,7 @@ class PipelineImpl(AbstractRedis[AnyStr], metaclass=PipelineMeta):
             try:
                 response.append(
                     cmd.callback(
-                        await self.parse_response(
-                            connection, cmd.command, **cmd.options
-                        ),
+                        await cmd.request if cmd.request else None,
                         **cmd.options,
                     )
                 )
@@ -558,7 +570,7 @@ class PipelineImpl(AbstractRedis[AnyStr], metaclass=PipelineMeta):
     ) -> None:
         assert isinstance(response, list)
         for i, r in enumerate(response):
-            if isinstance(r, ResponseError):
+            if isinstance(r, RedisError):
                 self.annotate_exception(r, i + 1, commands[i].command, commands[i].args)
                 raise r
 
@@ -580,38 +592,6 @@ class PipelineImpl(AbstractRedis[AnyStr], metaclass=PipelineMeta):
             )
             exception.args = (msg,) + exception.args[1:]
 
-    async def _parse(
-        self,
-        connection: BaseConnection,
-        command_name: bytes,
-        **options: Optional[ValueT],
-    ) -> ResponseType:
-        "Parses a response from the Redis server"
-        decode = options.get("decode")
-        response = await connection.read_response(
-            decode=decode if decode is None else bool(decode)
-        )
-        if command_name in self.response_callbacks:
-            callback = self.response_callbacks[command_name]
-            response = callback(response, **options)
-
-        return response
-
-    async def parse_response(
-        self,
-        connection: BaseConnection,
-        command_name: bytes,
-        **options: Optional[ValueT],
-    ) -> ResponseType:
-        result = await self._parse(connection, command_name, **options)
-
-        if command_name in self.UNWATCH_COMMANDS:
-            self.watching = False
-        elif command_name == CommandName.WATCH:
-            self.watching = True
-
-        return result
-
     async def load_scripts(self):
         # make sure all scripts that are about to be run on this pipeline exist
         scripts = list(self.scripts)
@@ -670,7 +650,7 @@ class PipelineImpl(AbstractRedis[AnyStr], metaclass=PipelineMeta):
 
             if self.watching:
                 raise WatchError(
-                    "A ConnectionError occured on while watching " "one or more keys"
+                    "A ConnectionError occured on while watching one or more keys"
                 )
             # otherwise, it's safe to retry since the transaction isn't
             # predicated on any state
@@ -703,7 +683,7 @@ class PipelineImpl(AbstractRedis[AnyStr], metaclass=PipelineMeta):
         )
 
 
-class ClusterPipelineImpl(AbstractRedis[AnyStr], metaclass=ClusterPipelineMeta):
+class ClusterPipelineImpl(Client[AnyStr], metaclass=ClusterPipelineMeta):
 
     connection_pool: ClusterConnectionPool
     command_stack: List[ClusterPipelineCommand]
@@ -715,8 +695,6 @@ class ClusterPipelineImpl(AbstractRedis[AnyStr], metaclass=ClusterPipelineMeta):
         self,
         connection_pool: ClusterConnectionPool,
         result_callbacks: Optional[Dict[bytes, Callable[..., Any]]] = None,
-        response_callbacks: Optional[Dict[bytes, Callable[..., Any]]] = None,
-        startup_nodes: Optional[List[Node]] = None,
         transaction: Optional[bool] = False,
         watches: Optional[Parameters[KeyT]] = None,
     ) -> None:
@@ -724,12 +702,11 @@ class ClusterPipelineImpl(AbstractRedis[AnyStr], metaclass=ClusterPipelineMeta):
         self.refresh_table_asap = False
         self.connection_pool: ClusterConnectionPool = connection_pool
         self.result_callbacks = result_callbacks
-        self.startup_nodes = startup_nodes if startup_nodes else []
-        self.response_callbacks = response_callbacks or {}
         self._transaction = transaction
         self.watches: Optional[Parameters[KeyT]] = watches or None
         self.watching = False
         self.explicit_transaction = False
+        self.cache = None  # not implemented.
 
     async def watch(self, *keys: KeyT) -> bool:
         raise NotImplementedError
@@ -833,7 +810,6 @@ class ClusterPipelineImpl(AbstractRedis[AnyStr], metaclass=ClusterPipelineMeta):
         self.command_stack = []
 
         self.scripts: Set[Script[AnyStr]] = set()
-        self.watches = []
         # clean up the other instance attributes
         self.watching = False
         self.explicit_transaction = False
@@ -854,13 +830,11 @@ class ClusterPipelineImpl(AbstractRedis[AnyStr], metaclass=ClusterPipelineMeta):
         if not slots:
             raise ClusterTransactionError("No slots found for transaction")
         node = self.connection_pool.get_node_by_slot(slots.pop())
-        conn = self.connection_pool.get_connection_by_node(node)
+        conn = await self.connection_pool.get_connection_by_node(node)
 
         if self.watches:
             await self._watch(node, conn, self.watches)
-        node_commands = NodeCommands(
-            self.parse_response, self.response_callbacks, conn, in_transaction=True
-        )
+        node_commands = NodeCommands(conn, in_transaction=True)
         node_commands.append(ClusterPipelineCommand(CommandName.MULTI, ()))
         node_commands.extend(attempt)
         node_commands.append(ClusterPipelineCommand(CommandName.EXEC, ()))
@@ -871,8 +845,8 @@ class ClusterPipelineImpl(AbstractRedis[AnyStr], metaclass=ClusterPipelineMeta):
             await node_commands.read()
         except ExecAbortError:
             if self.explicit_transaction:
-                await conn.send_command(CommandName.DISCARD)
-                await conn.read_response()
+                request = await conn.create_request(CommandName.DISCARD)
+                await request
 
         # If at least one watched key is modified before the EXEC command,
         # the whole transaction aborts,
@@ -908,33 +882,23 @@ class ClusterPipelineImpl(AbstractRedis[AnyStr], metaclass=ClusterPipelineMeta):
         # if we have to run through it again, we only retry the commands that failed.
         attempt = sorted(self.command_stack, key=lambda x: x.position)
 
+        protocol_version: int = 3
         # build a list of node objects based on node names we need to
         nodes: Dict[str, NodeCommands] = {}
-
         # as we move through each command that still needs to be processed,
         # we figure out the slot number that command maps to, then from the slot determine the node.
-
         for c in attempt:
             # refer to our internal node -> slot table that tells us where a given
             # command should route to.
             slot = self._determine_slot(c.command, *c.args)
             node = self.connection_pool.get_node_by_slot(slot)
 
-            # little hack to make sure the node name is populated. probably could clean this up.
-            self.connection_pool.nodes.set_node_name(node)
-
-            # now that we know the name of the node ( it's just a string in the form of host:port )
-            # we can build a list of commands for each node.
-            node_name = node["name"]
-
-            if node_name not in nodes:
-                nodes[node_name] = NodeCommands(
-                    self.parse_response,
-                    self.response_callbacks,
-                    self.connection_pool.get_connection_by_node(node),
+            if node.name not in nodes:
+                nodes[node.name] = NodeCommands(
+                    await self.connection_pool.get_connection_by_node(node),
                 )
 
-            nodes[node_name].append(c)
+            nodes[node.name].append(c)
 
         # send the commands in sequence.
         # we  write to all the open sockets for each node first, before reading anything
@@ -961,8 +925,8 @@ class ClusterPipelineImpl(AbstractRedis[AnyStr], metaclass=ClusterPipelineMeta):
         # next time we read from it we pass the buffered result back from a previous
         # command and every single request after to that connection will always get
         # a mismatched result. (not just theoretical, I saw this happen on production x.x).
-
         for n in nodes.values():
+            protocol_version = n.connection.protocol_version
             self.connection_pool.release(n.connection)
 
         # if the response isn't an exception it is a valid response from the node
@@ -1007,7 +971,7 @@ class ClusterPipelineImpl(AbstractRedis[AnyStr], metaclass=ClusterPipelineMeta):
         # turn the response back into a simple flat array that corresponds
         # to the sequence of commands issued in the stack in pipeline.execute()
         response = tuple(
-            c.callback(c.result, **c.options)
+            c.callback(c.result, version=protocol_version, **c.options)
             for c in sorted(self.command_stack, key=lambda x: x.position)
         )
 
@@ -1016,21 +980,6 @@ class ClusterPipelineImpl(AbstractRedis[AnyStr], metaclass=ClusterPipelineMeta):
 
         return response
 
-    async def parse_response(
-        self,
-        connection: ClusterConnection,
-        command_name: bytes,
-        **options: Any,
-    ) -> ResponseType:
-        """
-        Parses a response from the Redis server
-        :meta private:
-        """
-        decode = options.get("decode")
-        return await connection.read_response(
-            decode=decode if decode is None else bool(decode)
-        )
-
     def _determine_slot(self, command: bytes, *args: ValueT) -> int:
         """Figure out what slot based on command and args"""
 
@@ -1038,10 +987,10 @@ class ClusterPipelineImpl(AbstractRedis[AnyStr], metaclass=ClusterPipelineMeta):
 
         if not keys:
             raise RedisClusterException(
-                "No way to dispatch {} to Redis Cluster. Missing key".format(command)
+                f"No way to dispatch {command} to Redis Cluster. Missing key"
             )
 
-        slots = {self.connection_pool.nodes.keyslot(key) for key in keys}
+        slots = {hash_slot(b(key)) for key in keys}
 
         if len(slots) != 1:
             raise ClusterCrossSlotError(command=command, keys=keys)
@@ -1071,7 +1020,7 @@ class ClusterPipelineImpl(AbstractRedis[AnyStr], metaclass=ClusterPipelineMeta):
         raise RedisClusterException("method load_scripts() is not implemented")
 
     async def _watch(
-        self, node: Node, conn: BaseConnection, keys: Parameters[KeyT]
+        self, node: ManagedNode, conn: BaseConnection, keys: Parameters[KeyT]
     ) -> bool:
         "Watches the values at keys ``keys``"
 
@@ -1079,27 +1028,25 @@ class ClusterPipelineImpl(AbstractRedis[AnyStr], metaclass=ClusterPipelineMeta):
             slot = self._determine_slot(CommandName.WATCH, key)
             dist_node = self.connection_pool.get_node_by_slot(slot)
 
-            if node.get("name") != dist_node["name"]:
-                # raise error if commands in a transaction can not hash to same node
-
-                if len(node) > 0:
-                    raise ClusterTransactionError(
-                        "Keys in request don't hash to the same node"
-                    )
+            if node.name != dist_node.name:
+                raise ClusterTransactionError(
+                    "Keys in request don't hash to the same node"
+                )
 
         if self.explicit_transaction:
             raise RedisError("Cannot issue a WATCH after a MULTI")
-        await conn.send_command(CommandName.WATCH, *keys)
+        request = await conn.create_request(CommandName.WATCH, *keys)
 
         return SimpleStringCallback()(
-            cast(StringT, await conn.read_response()), version=conn.protocol_version
+            cast(StringT, await request),
+            version=conn.protocol_version,
         )
 
     async def _unwatch(self, conn: BaseConnection) -> bool:
         """Unwatches all previously specified keys"""
-        await conn.send_command(CommandName.UNWATCH)
-        res = cast(StringT, await conn.read_response())
-        return res in {"OK", b"OK"} if self.watching else True
+        request = await conn.create_request(CommandName.UNWATCH, decode=False)
+        res = cast(StringT, await request)
+        return res == b"OK" if self.watching else True
 
 
 class Pipeline(ObjectProxy, Generic[AnyStr]):  # type: ignore
@@ -1129,14 +1076,14 @@ class Pipeline(ObjectProxy, Generic[AnyStr]):  # type: ignore
     def proxy(
         cls,
         connection_pool: ConnectionPool,
-        response_callbacks: Dict[bytes, Callable[..., Any]],
         transaction: Optional[bool] = None,
+        watches: Optional[Parameters[KeyT]] = None,
     ) -> Pipeline[AnyStr]:
         return cls(
             PipelineImpl(
                 connection_pool,
-                response_callbacks=response_callbacks,
                 transaction=transaction,
+                watches=watches,
             )
         )
 
@@ -1171,6 +1118,10 @@ class Pipeline(ObjectProxy, Generic[AnyStr]):  # type: ignore
         return await self.__wrapped__.execute(raise_on_error=raise_on_error)
 
     async def reset(self) -> None:
+        """
+        Resets the command stack and releases any connections acquired from the
+        pool
+        """
         await self.__wrapped__.reset_pipeline()
 
 
@@ -1202,8 +1153,6 @@ class ClusterPipeline(ObjectProxy, Generic[AnyStr]):  # type: ignore
         cls,
         connection_pool: ClusterConnectionPool,
         result_callbacks: Optional[Dict[bytes, Callable[..., Any]]] = None,
-        response_callbacks: Optional[Dict[bytes, Callable[..., Any]]] = None,
-        startup_nodes: Optional[List[Node]] = None,
         transaction: Optional[bool] = False,
         watches: Optional[Parameters[KeyT]] = None,
     ) -> ClusterPipeline[AnyStr]:
@@ -1211,8 +1160,6 @@ class ClusterPipeline(ObjectProxy, Generic[AnyStr]):  # type: ignore
             ClusterPipelineImpl(
                 connection_pool,
                 result_callbacks=result_callbacks,
-                response_callbacks=response_callbacks,
-                startup_nodes=startup_nodes,
                 transaction=transaction,
                 watches=watches,
             )
@@ -1239,4 +1186,8 @@ class ClusterPipeline(ObjectProxy, Generic[AnyStr]):  # type: ignore
         return await self.__wrapped__.execute(raise_on_error=raise_on_error)
 
     def reset(self) -> None:
+        """
+        Resets the command stack and releases any connections acquired from the
+        pool
+        """
         self.__wrapped__.reset_pipeline()
